//*****************************************************************************
//                    Laborprojekt, FH-Wedel 2005
//-----------------------------------------------------------------------------
//
// Modul:       - enthlt Routinen zur Auswertung eines empfangenen USB-Pakets
//              - ist fr smtliche Flash-Vorgnge zustndig wie z.B. Lschen,
//                Schreiben, Auslesen, Chip-Typ ermitteln etc.
//
//-----------------------------------------------------------------------------
// Datum:       Copyright 2005, Andreas Schibilla
//*****************************************************************************
#include "at89c5131.h"
#include "ext_5131.h"
#include "compiler.h"
#include "usb.h"    
#include "commands.h"
#include "flash.h"


// ---Zuordnung der Controller-Pins:
//
//      <AT89C4051>       <AT89C5131>
#define C4051_XTAL1       P1_1

#define C4051_P3_1        P3_7       
#define C4051_P3_2        P1_0
#define C4051_P3_3        P1_2
#define C4051_P3_4        P2_1
#define C4051_P3_5        P2_0
#define C4051_P3_7        P1_4
#define C4051_P1          P0
#define C4051_RST_TR_IN   P3_4
#define C4051_RST_SHDN    P2_2  
                           
#define C_PROG_READY      0                    
#define C_PROG_BURN       1
#define C_PROG_VERIFY     2
              

//--- globale Variablen                          
unsigned char g_signature[2]    = { 0,0 };             // enthlt die beiden ausgelesenen Signatur-Bytes
unsigned int  g_flash_size      = 1024;                // enthlt die Flashgre des eingeleten Chips
unsigned int  g_addr            = 0x0000;              // Adresszhler
unsigned char g_prog_mode       = C_PROG_READY;        // ProgMode: Soll Programmiert oder Verifiziert werden?
    

            
//*****************************************************************************
// Funktion:    Wartet n Millisekunden
//              fr 1ms: 1333.3 Takte verbraten (bei 16MHz Quarz, 12CLK/OP)
//              Aufgrund des Funktionsaufrufs und dem Durchlaufen der
//              while-Schleife werden noch mehr Takte verbraten -> ungenau!
//-----------------------------------------------------------------------------
// Anmerkung:   Die Funktion geht von 12 Clocks per Instruction aus, auch
//              wenn im Datenblatt 6 angegeben ist (Fehler???)
//-----------------------------------------------------------------------------
// Parameter:   len  - Anzahl ms die gewartet werden soll
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-23, Andreas Schibilla
//*****************************************************************************    
void wait_ms(unsigned char len)
{
  
 while (len)
 {  
      // Warteschleife: 1 ms
      // Anzahl Maschinenzyklen: 1333.3333333333333
      _asm 
      00001$:
        push psw
        push 0
        mov  0,#188
      00002$:
        nop
        nop
        nop
        nop
        nop
        djnz 0,00002$
        nop
        nop
        nop
        nop
        pop 0
        pop psw
      _endasm ;  
      
      // wieder eine ms gewartet
      --len; 
 }     
}    


   
//*****************************************************************************
// Funktion:    Wartet n Mikrosekunden
//              fr 1us: 1,3 Takte (bei 16MHz Quarz, 12CLK/OP) 
//              Aufgrund des Funktionsaufrufs und dem Durchlaufen der
//              while-Schleife werden noch mehr Takte verbraten -> ungenau!
//-----------------------------------------------------------------------------
// Anmerkung:   Die Funktion geht von 12 Clocks per Instruction aus, auch
//              wenn im Datenblatt 6 angegeben ist (Fehler???)
//-----------------------------------------------------------------------------
// Parameter:   len  - Anzahl ms die gewartet werden soll
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-23, Andreas Schibilla
//*****************************************************************************
void wait_us(unsigned char len)
{ 
 while (len)
 {  
      _asm 
        nop
        nop
      _endasm ;  
      
      // wieder eine us gewartet
      --len; 
 }      
}


    
//*****************************************************************************
// Funktion:    Setzt die Spannung am Reset Port auf 12V
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-20, Andreas Schibilla
//*****************************************************************************
void set_reset_12v(void)
{
 C4051_RST_TR_IN  = 1;
 C4051_RST_SHDN   = 0;
}



//*****************************************************************************
// Funktion:    Setzt die Spannung am Reset Port auf 5V
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-20, Andreas Schibilla
//*****************************************************************************
void set_reset_5v(void)
{
 C4051_RST_TR_IN  = 1;
 C4051_RST_SHDN   = 1;  
}



//*****************************************************************************
// Funktion:    Setzt die Spannung am Reset Port auf 0V
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-20, Andreas Schibilla
//*****************************************************************************
void set_reset_0v(void)
{
 C4051_RST_TR_IN  = 0;
 C4051_RST_SHDN   = 1;
}
 
 

//*****************************************************************************
// Funktion:    Initialisiert den Chip - Power-Up-Sequence
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-23, Andreas Schibilla
//*****************************************************************************
void chip_init(void)
{
 set_reset_0v();
 C4051_XTAL1 = 0;   
 wait_us(10);
 
 set_reset_5v();
 C4051_P3_2  = 1;
 wait_us(10);
}
 
 
 
//*****************************************************************************
// Funktion:    Erhht die Adresse durch Pulse an XTAL1
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-23, Andreas Schibilla
//*****************************************************************************
void inc_addr(void)
{
 C4051_XTAL1 = 1;
 
 // min 200ns warten
 _asm 
        nop
 _endasm ;  
       
 C4051_XTAL1 = 0;
} 



//*****************************************************************************
// Funktion:    Liest die Signaturbytes des Chips aus (Hersteller+Produkt),
//              speichert die entspr. Flashgre global ab (in g_flash_size)
//              und schickt das Ergebnis via USB an den PC
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      g_flash_size - bekommt die ermittelte Flashgre!
//-----------------------------------------------------------------------------
// Datum:       2005-06-23, Andreas Schibilla
//*****************************************************************************
void chip_read_signature(void)
{                       
 C4051_P1    = 0xFF;   // Port 1 zum Lesen vorbereiten ->Input

 chip_init(); 
                          
 C4051_P3_3  = 0;
 C4051_P3_4  = 0;
 C4051_P3_5  = 0;
 C4051_P3_7  = 0;
 wait_us(3);
 
 g_signature[0] = C4051_P1;
 wait_us(1);
 
 inc_addr();

 g_signature[1] = C4051_P1;

 // Flashgre des Chips ermitteln und global abspeichern
 switch (g_signature[1])
 {
    case 0x21:  g_flash_size = 2048;
                break;
    case 0x41:  g_flash_size = 4096;
                break;
    case 0x11:
    case 0x12:
    default:    g_flash_size = 1024;
                break;  
 }

 usb_send_data(PACKET_SIG, 0, g_signature, 2);
}
        

        
//*****************************************************************************
// Funktion:    lscht den Chip und schickt anschlieend eine Besttigung
//              via USB an den PC
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-23, Andreas Schibilla
//*****************************************************************************
void chip_erase(void)
{                       
 chip_init();
 
 C4051_P3_3  = 1;
 C4051_P3_4  = 0;
 C4051_P3_5  = 0;
 C4051_P3_7  = 0;
 wait_us(1);
 
 set_reset_12v();
 wait_us(50);

 C4051_P3_2  = 0;
 wait_us(1);                         // t_GLGH
 wait_ms(20);

 C4051_P3_2  = 1;
 wait_us(10);                        // t_GHSL
 set_reset_5v();
 wait_us(1);
                                                         
 usb_send_data(PACKET_SIG, 0, "ERASE", sizeof("ERASE") );
}
   


//*****************************************************************************
// Funktion:    berprft ob das Lschen des Chips erfolgreich war, also
//              alle Bytes 0xFF sind. Das Ergebnis wird anschlieend via
//              USB zum PC gesendet!
//-----------------------------------------------------------------------------
// Parameter:   flash_size - Gre des Flash-Speichers des Chips
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-23, Andreas Schibilla
//*****************************************************************************
void chip_blank_check(int flash_size)
{   
 unsigned char dat; 
 unsigned int i;
 unsigned char err[3];
  
 chip_init();
 C4051_P1 = 0xFF;   // Port 1 zum Lesen vorbereiten ->Input

 C4051_P3_3  = 0;
 C4051_P3_4  = 0;
 C4051_P3_5  = 1;
 C4051_P3_7  = 1;
 wait_us(1);
 
 for (i=0; i<flash_size; ++i)
 {                            
    dat = C4051_P1;    // Wert lesen

    if (dat != 0xFF)
    {   
        // Fehler! Adresse, bei der Fehler aufrat, berechnen
        err[0] = LOW(i);
        err[1] = HIGH(i);
        err[2] = dat;                                      
        // Fehlerpaket mit Adresse, bei der Fehler auftrat, zurckschicken
        LED_ERROR = 0;  // Fehler LED
        usb_send_data(0, 0, &err, 3);
        return; 
    }
    inc_addr();
 }

 usb_send_data(PACKET_SIG, 0, "OK", sizeof("OK") );  
}
 
 


//*****************************************************************************
// Funktion:    DEBUG-Funktion!!!
//              Liest 64 Bytes aus dem Chip aus und sendet sie direkt in
//              einem USB-Bulk-Paket zum PC
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-20, Andreas Schibilla
//*****************************************************************************
#define Usb_tx_complete()             (UEPSTAX & MSK_TXCMPL)
#define Usb_clear_tx_ready()          (UEPSTAX &= ~MSK_TXRDY)
#define Usb_clear_tx_complete()       (UEPSTAX &= ~MSK_TXCMPL)
#define Usb_set_tx_ready()            (UEPSTAX |= MSK_TXRDY)

void chip_debug(void)
{
 unsigned char i,c;
  
 chip_init();
 C4051_P1 = 0xFF;     // Port 1 zum Lesen vorbereiten ->Input

 C4051_P3_3  = 0;
 C4051_P3_4  = 0;
 C4051_P3_5  = 1;
 C4051_P3_7  = 1;
 wait_us(1);  
 
    
 UEPNUM = 5;          // EP5 auswhlen
 for (i=0; i<64; ++i)
 {
    c = C4051_P1;     // Wert lesen
    UEPDATX = c;
    inc_addr();
 }
 
 Usb_set_tx_ready();
 while (!Usb_tx_complete());
 Usb_clear_tx_complete(); 
}
  


//*****************************************************************************
// Funktion:    Initialisiert (Power-Up-Sequence) den Chip fr den folgenden
//              Brenn- oder Verifiziervorgang und schickt Besttigung via
//              USB zum PC!
//-----------------------------------------------------------------------------
// Parameter:   prog_mode - gibt an, ob der Chip gebrannt (C_PROG_BURN)
//                          oder verifiziert (C_PROG_VERIFY) wird 
// Return:      speichert den Mode global ab (g_prog_mode) und setzt die 
//              globale Adressvar. zurck (g_addr)
//-----------------------------------------------------------------------------
// Datum:       2005-06-25, Andreas Schibilla
//*****************************************************************************
void chip_init_burn_or_verify(unsigned char prog_mode)
{
 chip_init();
 
 C4051_P3_3  = 0;
 C4051_P3_5  = 1;
 C4051_P3_7  = 1;
 
 // soll gebrannt oder verifiziert werden?
 if ( prog_mode == C_PROG_BURN )
 {
    C4051_P1     = 0x00;                // Low-Pegel anlegen, damit Daten gebrannt werden knnen
    C4051_P3_4   = 1; 
    set_reset_12v();                    // 12V fr Brennen bereitstellen 
    C4051_P3_1   = 1;
    wait_us(10);                        // t_SHGL 
 }
 else
 {                                               
    C4051_P3_4  = 0; 
    wait_us(10);
    C4051_P1 = 0xFF;                    // Port 1 zum Lesen vorbereiten ->Input
 }
  
 g_prog_mode  = prog_mode;              // Verify oder Burn global merken
 g_addr       = 0x0000;                 // Adresse wieder bei 0

 // Besttigung via USB an PC senden, HEXDATA kann folgen 
 usb_send_data(PACKET_SIG, 0, "OK", sizeof("OK"));  
}  



//*****************************************************************************
// Funktion:    berprft, ob ein Paket gltigen Hex-Record enthlt
//-----------------------------------------------------------------------------
// Parameter:   p     -  Zeiger auf Anfang der Paketbytes
//              num   -  Anzahl der Bytes des Pakets
// Return:      liefert 1, wenn alles ok
//              liefert 0, wenn ungltiger befehl
//-----------------------------------------------------------------------------
// Datum:       2005-06-25, Andreas Schibilla
//*****************************************************************************
unsigned char chip_hex_record_ok(char *p, unsigned char num)
{ 
  
 // ist empfangenes Paket lang genug?
 if ( (num < 3) || (num < (6 + p[2])) )      // nicht strikte Auswertung ist wichtig!!!
 {
    // Fehler 
    LED_ERROR = 0;  // Fehler LED
    usb_send_data(0, 0, &num, 1); 
    return 0;
 }
 
 // welcher Hex-Record-Typ ist im Paket?
 switch (p[5])
 {  
    // Daten
    case 0x00:    return 1;   
      
    // EOF
    case 0x01:    wait_us(10);                        // t_GHSL
                  set_reset_5v();
                  usb_send_data(PACKET_SIG, 0, "OK", sizeof("OK"));         
                  return 0;
      
    // Fehler: Ungltiger Typ
    default:      LED_ERROR = 0;  // Fehler LED
                  usb_send_data(PACKET_SIG, CMD_ERROR_STR, "ERR: Ungltiger Intel-Hex-Typ!", sizeof("ERR: Ungltiger Intel-Hex-Typ!"));
                  return 0;
       
 }

}



//*****************************************************************************
// Funktion:    Brennt oder Verifiziert EINEN Hex-Record
//-----------------------------------------------------------------------------
// Parameter:   p     -  Zeiger auf Anfang der Paketbytes
//              num   -  Anzahl der Bytes des Pakets
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-25, Andreas Schibilla
//*****************************************************************************
void chip_burn_or_verify(char *p, unsigned char num)
{ 
 unsigned int  intel_addr;
 unsigned char i, dat;
 unsigned char *c;  
 
 // Lnge und Typ des Hex-Records okay?
 if (!chip_hex_record_ok(p, num) )
    return;  
                                     
 // Intel-Adresse aus dem Hex-Record ermitteln
 c = (unsigned char *) &intel_addr;
 c[0] = p[4];
 c[1] = p[3];
 
 // ist Intel-Adresse gltig?
 if ( (intel_addr+p[2]) > g_flash_size )
 {   
    // Fehler, Adresse zu gro fr den Chip!!! 
    LED_ERROR = 0;  // Fehler LED
    usb_send_data(PACKET_SIG, CMD_ERROR_STR, "ERR: Adresse grer als Flashspeicher!", sizeof("ERR: Adresse grer als Flashspeicher!"));
    return;        
 }
 if ( intel_addr < g_addr )
 {   
    // Fehler, Ungltige Adresse, da waren wir schon
    LED_ERROR = 0;  // Fehler LED
    usb_send_data(PACKET_SIG, CMD_ERROR_STR, "ERR: Ungltige Adresse! Kann nicht zurcklaufen", sizeof("ERR: Ungltige Adresse! Kann nicht zurcklaufen"));
    return;        
 }
  
 // bis zur richtigen Adresse vorgehen
 while (intel_addr > g_addr)
 {
    inc_addr();    
    ++g_addr;
 }
    
 // Daten brennen oder verifizieren
 c = p+6;
 for (i=0; i<p[2]; ++i)
 {   
    if(g_prog_mode == C_PROG_BURN)
    {
        C4051_P1 = c[i];                    // Daten anlegen
        wait_us(1);                         // t_DVGL
        C4051_P3_2  = 0;
        wait_us(1);                         // t_GLGH
        C4051_P3_2  = 1;
        wait_us(1);                         // t_GHBL                
        while(C4051_P3_1 == 0) { ; }        // warten von BUSY bis READY
        wait_us(1);                         // t_BHIH
    } 
    else 
    {   
        dat = C4051_P1;
        if (dat != c[i])        // Vergleichen, ob Byte korrekt
        {
            // Fehler
            LED_ERROR = 0;  // Fehler LED
            usb_send_data(PACKET_SIG, CMD_ERROR_STR, "ERR: Bytes nicht identisch!", sizeof("ERR: Bytes nicht identisch!"));
            return;        
        }
          
    }     
    inc_addr();
    ++g_addr;  
 }
    
 // Besttigung via USB an PC senden
 usb_send_data(PACKET_SIG, 0, "OK", sizeof("OK")); 
 
}




//*****************************************************************************
// Funktion:    Schreibt die Lockbits in den Chip
//-----------------------------------------------------------------------------
// Parameter:   lockbits  - gibt an, wie die Lockbits gesetzt werden sollen
//              0: No program lock features
//              1: Further programming of the Flash is disabled
//              3: Same as 1, also verify is disabled
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-29, Andreas Schibilla
//*****************************************************************************
void chip_write_lockbits(unsigned char lockbits)
{                       
 chip_init();
 set_reset_12v();                    // 12V fr Brennen bereitstellen
 wait_us(10); 
                             
 if ( (lockbits==1) || (lockbits==2) )
 {
    // Lockbit 1 schreiben: 
    C4051_P3_3  = 1;
    C4051_P3_4  = 1;
    C4051_P3_5  = 1;
    C4051_P3_7  = 1;
    wait_us(2);
    C4051_P3_2  = 0;
    wait_us(2);                         // t_GLGH
    C4051_P3_2  = 1;
    wait_us(1);                         // t_GHBL        
    while(C4051_P3_1 == 0) { ; }        // warten von BUSY bis READY
    wait_us(1);                         // t_BHIH 
 }
 
 if (lockbits==2)
 {
    // Lockbit 2 schreiben: 
    C4051_P3_3  = 1;
    C4051_P3_4  = 1;
    C4051_P3_5  = 0;
    C4051_P3_7  = 0;
    wait_us(2);
    C4051_P3_2  = 0;
    wait_us(2);                         // t_GLGH
    C4051_P3_2  = 1;
    wait_us(1);                         // t_GHBL        
    while(C4051_P3_1 == 0) { ; }        // warten von BUSY bis READY
    wait_us(1);                         // t_BHIH 
 }
 

 // Besttigung via USB an PC senden
 usb_send_data(PACKET_SIG, 0, "OK", sizeof("OK")); 
}



//*****************************************************************************
// Funktion:    Wertet ein empfangenes USB-Paket aus, interpretiert den
//              Befehl, fhrt ihn aus und sendet das Ergebnis zum PC 
//-----------------------------------------------------------------------------
// Parameter:   p     -  Zeiger auf Anfang der Paketbytes
//              num   -  Anzahl der Bytes des Pakets
// Return:      liefert 1, wenn alles ok
//              liefert 0, wenn ungltiger befehl
//-----------------------------------------------------------------------------
// Datum:       2005-06-15, Andreas Schibilla
//*****************************************************************************
int usb_eval(char *p, unsigned char num)
{
 LED_ERROR = 1;     // kein Fehler, evtl. vorhanden Fehler clearen
 
 if (num < PACKET_HEADER_LEN)  // hat das Paket die Mindestlnge?
 {    
      LED_ERROR = 0;  // Fehler LED
      usb_send_data(PACKET_SIG, CMD_ERROR_STR, C_ERR_PACKETLENGTH, sizeof(C_ERR_PACKETLENGTH));
      return 0;
 }
 
 
 // Was fr ein Paket ist das?
 if (p[0] != PACKET_SIG)      // ungltige Paket-Signatur?
 {      
      LED_ERROR = 0;  // Fehler LED
      usb_send_data(PACKET_SIG, CMD_ERROR_STR, C_ERR_SIGNATURE, sizeof(C_ERR_SIGNATURE));        
      return 0;
 }    
               
 // Welcher Befehl ist angekommen?               
 switch (p[1])
 {          
          case CMD_ERASE:           chip_erase();
                                    break;   
                                   
          case CMD_BLANK_CHECK:     chip_blank_check(g_flash_size);
                                    break;   

          case CMD_DEBUG:           chip_debug();
                                    break;   
                                    
          case CMD_PROGRAM:         chip_init_burn_or_verify(C_PROG_BURN);
                                    break;
                                    
          case CMD_VERIFY:          chip_init_burn_or_verify(C_PROG_VERIFY);
                                    break;  
                                    
          case CMD_SET_LOCK:        chip_write_lockbits(p[2]);
                                    break;
        
          case CMD_GET_VERSION:     usb_send_data(PACKET_SIG, 0, C_VERSION, sizeof(C_VERSION));
                                    break;                                                                      
                                    
          case CMD_GET_TYPE:        chip_read_signature();
                                    break;
                                                
          case CMD_HEXDATA:         chip_burn_or_verify(p, num);
                                    break; 
            
          default:                  LED_ERROR = 0;  // Fehler LED
                                    usb_send_data(PACKET_SIG, CMD_ERROR_STR, C_ERR_UNKNOWN, sizeof(C_ERR_UNKNOWN));
                                    break;
        
 }
 
 return 0; 
}
