//*****************************************************************************
//                    Laborprojekt, FH-Wedel 2005
//-----------------------------------------------------------------------------
//
// Programm:    - Steuerprogramm zum USB Programmer fr AT89C2051 / AT89C4051
//              - Konsolenanwendung, die das automatische Programmieren
//                ermglicht, indem per Kommandozeile der Name der Hex-Datei
//                bergeben wird
//
//-----------------------------------------------------------------------------
// Datum:       Copyright 2005, Andreas Schibilla
//*****************************************************************************
#include <windows.h>
#include <stdio.h>
#include "usb.h"
#include "commands.h"
#include "log.h"


#pragma comment (lib,"shlwapi.lib")

int main(int argc, char* argv[])
{
 t_signature sig;
 DWORD       dwTime = GetTickCount(); 
 BOOL        bIsHexFile = FALSE;
 

 //--- berprfe, ob Kommandozeile genau einen Parameter enthlt
 if (argc != 2)
 {
        printf("\r\nUngltiger Aufruf!!!");
        printf("\r\n====================\r\n");
        printf("\r\nSyntax: usb_programmer.exe hexfile\r\n");
        printf("\r\nhexfile kann eine Intel-Hex-Datei sein (Datei-Extension: .hex oder .ihx)");
        printf("\r\n        oder eine Binrdatei           (andere Datei-Extension)\r\n\r\n\r\n");
        return 1;
 }

 
 //--- ermittle Chip-Typ und Programmerversion
 if ( chip_type_and_version(&sig) )
 {
        char szMfr[64], szArt[64], szVersion[64];

        strcpy(szMfr,       sig.mfr_str);
        strcpy(szArt,       sig.art_str);
        strcpy(szVersion,   sig.version_str);

        printf("      Version:     %s\r\n",    szVersion);
        printf("      Hersteller:  %s\r\n", szMfr);
        printf("      Produkt:     %s\r\n",    szArt);

 }
 else
 {
        return 1;
 }


 //--- lsche den Chip
 if (chip_erase() )
 {
    printf("      Chip gelscht...\r\n");
 }
 else
 {
        return 1;
 }


 //--- Blank Check
 if (chip_blank_check() )
 {
    printf("      Blank Check erfolgreich...\r\n");
 }
 else
 {
        return 1;
 }
 
 bIsHexFile = isHexFile(argv[1]);
 
 //--- Flash-Programmierung
 if (chip_program(argv[1], bIsHexFile, CMD_PROGRAM) )
 {
    printf("      Flash-Programmierung abgeschlossen...\r\n");
 }
 else
 {
        return 1;
 }


 //--- Verifizierung
 if (chip_program(argv[1], bIsHexFile, CMD_VERIFY ) )
 {
    printf("      Flash-Verifizierung erfolgreich...\r\n");
 }
 else
 {
        return 1;
 }

 dwTime = GetTickCount() - dwTime;
 double dTime = (double) dwTime / 1000.0f;
 printf("\r\nDauer: %.2f Sekunden", dTime);                                               

 return 0;
}
